#!/usr/bin/env node

const { execFileSync } = require('child_process');
const { readFileSync, writeFileSync, existsSync } = require('fs');
const path = require('path');
const readline = require('readline').createInterface({
	input: process.stdin,
	output: process.stdout,
});

run().finally(() => {
	readline.close();
});

async function run() {
	const customElementName = await getCustomElementName();

	if (!customElementName) {
		return;
	}

	const directoryName = await getDirectoryName(customElementName);

	if (!directoryName) {
		return;
	}

	degitOfficialSvelteComponentTemplate(directoryName);

	updatePackageJson(
		path.join(directoryName, 'package.json'),
		customElementName
	);

	updateRollupConfig(
		path.join(directoryName, 'rollup.config.js'),
		customElementName
	);

	createRollupTestConfig(path.join(directoryName, 'rollup.test.config.js'));

	createTestHtml(
		path.join(directoryName, 'index.html'),
		path.join(directoryName, 'src', 'index.svelte'),
		customElementName
	);

	createSvelteComponent(path.join(directoryName, 'src', 'index.svelte'));

	npmInstall(directoryName);

	showSuccessMessage(directoryName);
}

async function getCustomElementName() {
	let customElementName = process.argv[2]
		? fixCustomElementName(path.basename(process.argv[2]))
		: 'my-svelte-component';

	const chosenCustomElementName = await question(
		`We're going to register a custom element for your component,
so you can use it like this:

	<my-svelte-component></my-svelte-component>

What would you like your tag name to be? [${customElementName}]: `
	);

	if (chosenCustomElementName) {
		customElementName = fixCustomElementName(chosenCustomElementName);

		const yesOrNo = await question(`We fixed the tag name for you to keep it valid:

		<${customElementName}></${customElementName}>

Is this okay? [Y/n]: `);

		if (/^(y|$)/i.test(yesOrNo) === false) {
			console.log('Okay, please try again.');
			return;
		}
	}

	return customElementName;
}

async function getDirectoryName(customElementName) {
	if (process.argv[2]) {
		return process.argv[2];
	}

	const directoryName =
		(await question(
			`What is the new directory name you want to use? [${customElementName}]: `
		)) || customElementName;

	if (existsSync(directoryName)) {
		console.log(
			`Looks like there's already a ${directoryName} directory, please try again.`
		);
		return;
	}

	return path.normalize(directoryName);
}

async function degitOfficialSvelteComponentTemplate(directoryName) {
	execFileSync(`npx`, [
		`degit`,
		`sveltejs/component-template`,
		directoryName,
	]);
}

function updatePackageJson(packageJsonPath, customElementName) {
	const pkg = JSON.parse(readFileSync(packageJsonPath));

	pkg.name = customElementName;
	pkg.scripts.test = 'rollup -c rollup.test.config.js --watch';

	writeFileSync(packageJsonPath, JSON.stringify(pkg, null, ' '));
}

function updateRollupConfig(rollupConfigPath, customElementName) {
	const rollupConfig = String(readFileSync(rollupConfigPath));

	writeFileSync(
		rollupConfigPath,
		rollupConfig.replace(
			'svelte()',
			`svelte({ customElement: true, tag: '${customElementName}' })`
		)
	);
}

function createRollupTestConfig(rollupTestConfigPath) {
	writeFileSync(
		rollupTestConfigPath,
		`import config from './rollup.config.js';
import serve from 'rollup-plugin-serve';
import livereload from 'rollup-plugin-livereload';

export default {
    input: config.input,
    output: config.output,
    plugins: config.plugins.concat([
		serve({
			contentBase: '',
			open: true
		}),
		livereload()
	])
};
`
	);
}

function createTestHtml(testHtmlPath, svelteComponentPath, customElementName) {
	writeFileSync(
		testHtmlPath,
		`<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8" />
        <title>${customElementName} Test</title>
        <script src="../index.js"></script>
    </head>
    <body>
		<${customElementName} message="Welcome to Svelte!">
			<p>Great job! Next steps:</p>
			<ol>
				<li>View source on this web page!</li>
				<li>Try editing the Svelte component at: <strong>${svelteComponentPath}</strong></li>
				<li>Try editing the HTML file at <strong>${testHtmlPath}</strong></li>
				<li>Run <strong>npm run build</strong> in the terminal to build your components.</li>
				<li>Use with other projects by installing this npm package, or copying over the file <strong>index.js</strong>.</li>
				<li>Use your custom element in any HTML or JSX code: <strong>&lt;${customElementName}&gt;&lt;/${customElementName}&gt;</strong>
			</ol>
		</${customElementName}>
    </body>
</html>`
	);
}

function createSvelteComponent(svelteComponentPath) {
	writeFileSync(
		svelteComponentPath,
		`<script>
export let message;
</script>

<h1>{message}</h1>

<slot/>`
	);
}

function npmInstall(directoryName) {
	execFileSync('npm', ['install'], {
		cwd: directoryName,
	});

	execFileSync(
		'npm',
		[
			'install',
			'--save-dev',
			'rollup-plugin-serve',
			'rollup-plugin-livereload',
		],
		{
			cwd: directoryName,
		}
	);
}

function showSuccessMessage(directoryName) {
	console.log(`*********************************************************************

We did it! To start working on your new component:

	cd ${directoryName}
	npm test

Start editing your src/index.svelte and index.html files
and see your changes show up automatically in the browser.

When you build with "npm test" or "npm run build", you'll also get:
	- index.js, to use in all browsers and node.js (UMD)
	- index.mjs, to use with <script type="module"> or node.js (ESM)

Thanks, and have fun!`);
}

function question(q) {
	return new Promise(resolve => {
		readline.question(q, resolve);
	});
}

function fixCustomElementName(name) {
	name = name
		.toLowerCase()
		.replace(/[^a-z0-9-]/g, '-')
		.replace(/-+/g, '-');

	if (/-/.test(name)) {
		return name;
	}

	return `x-${name}`;
}
